'use strict';

const path = require('path');
const fg = require('fast-glob');
const scule = require('scule');
const localPkg = require('local-pkg');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

const path__default = /*#__PURE__*/_interopDefaultCompat(path);
const fg__default = /*#__PURE__*/_interopDefaultCompat(fg);

function deleteArrayItem(arr, item) {
  const index = arr.findIndex((v) => v === item);
  if (index !== -1) {
    arr.splice(index, 1);
  }
}
function showModule(path$1) {
  return path.basename(path$1, path.extname(path$1));
}

function createHooks() {
  const effects = /* @__PURE__ */ new Map();
  function keys() {
    return Array.from(effects.keys());
  }
  function track(dir, effect) {
    effects.set(dir, effect);
  }
  function trigger(path$1, event) {
    const target = path.dirname(path$1);
    const effect = effects.get(target);
    if (effect) {
      const module = showModule(path$1);
      effect(event, module);
    }
  }
  return {
    keys,
    track,
    trigger
  };
}

const { track, trigger, keys } = createHooks();
function generateModules(options) {
  const { target, prefix, suffix, include, exclude } = options;
  const scanDirInInit = path__default.posix.resolve(target);
  const existedModulesInInit = fg__default.sync(`${scanDirInInit}/**/*`).map(showModule);
  const modules = /* @__PURE__ */ new Set([
    ...include,
    ...existedModulesInInit
  ]);
  track(
    path__default.resolve(target),
    // track module update logic
    function(event, module) {
      if (event === "add") {
        const hasPrefix = module.startsWith(prefix);
        const hasSuffix = module.endsWith(suffix);
        const shouldAppend = hasPrefix && hasSuffix && !exclude.includes(module);
        if (shouldAppend) {
          modules.add(module);
        }
      }
      if (event === "unlink") {
        if (include.includes(module)) {
          return;
        }
        if (modules.has(module)) {
          modules.delete(module);
        }
      }
    }
  );
  return modules;
}
function defaultNormalize({ path: path2 }) {
  return path2;
}
function dirResolver(options) {
  const {
    target = "src/composables",
    suffix = "",
    prefix = "",
    include = [],
    exclude = [],
    normalize = defaultNormalize
  } = options || {};
  const modules = generateModules({
    target,
    suffix,
    prefix,
    include,
    exclude
  });
  return function(name) {
    if (modules.has(name)) {
      return normalize({
        name,
        target,
        path: path__default.posix.resolve(target, name)
      });
    }
    name = scule.kebabCase(name);
    if (modules.has(name)) {
      return normalize({
        name,
        target,
        path: path__default.posix.resolve(target, name)
      });
    }
  };
}
function DirResolverHelper() {
  return {
    enforce: "pre",
    name: "vite-auto-import-resolvers:dir-resolver-helper",
    configureServer({ watcher }) {
      watcher.add(keys());
      watcher.on("add", (path2) => {
        trigger(path2, "add");
      });
      watcher.on("unlink", (path2) => {
        trigger(path2, "unlink");
      });
    }
  };
}

function AutoGenerateImports(options) {
  const {
    include = [
      "ahooks",
      "@vueuse/core",
      "@vueuse/math",
      "@vueuse/head",
      "mobx",
      "mobx-react-lite",
      "preact",
      "quasar",
      "react",
      "react-router",
      "react-router-dom",
      "react-i18next",
      "svelte",
      "svelte/animate",
      "svelte/easing",
      "svelte/motion",
      "svelte/store",
      "svelte/transition",
      "vee-validate",
      "vitepress",
      "vue-router",
      "vue-router/composables",
      "vuex",
      "uni-app",
      "solid-js",
      "@solidjs/router",
      "solid-app-router",
      "jotai",
      "jotai/utils",
      "recoil",
      "@vue/composition-api",
      "pinia",
      "vue-demi",
      "vue-i18n",
      "vue-router-composables",
      "vue",
      "vue/macros",
      "vitest"
    ],
    exclude = []
  } = options || {};
  const presets = include.filter((preset) => {
    if (exclude.includes(preset)) {
      return false;
    }
    return localPkg.isPackageExists(preset);
  });
  if (presets.includes("vue") && presets.includes("vue-demi")) {
    deleteArrayItem(presets, "vue-demi");
  }
  return presets;
}
const vue3Presets = [
  "vue",
  "vuex",
  "pinia",
  "vue-i18n",
  "vue-router",
  "@vueuse/core",
  "@vueuse/head",
  "@vueuse/math"
];

exports.AutoGenerateImports = AutoGenerateImports;
exports.DirResolverHelper = DirResolverHelper;
exports.dirResolver = dirResolver;
exports.vue3Presets = vue3Presets;
